@@ -1,32 +1,34 @@
 /*
- * sip-connection-private.h - Private structures for SIPConnection
+ * sip-connection-private.h - Private structures for TpsipConnection
  * Copyright (C) 2005-2007 Collabora Ltd.
- * Copyright (C) 2005-2007 Nokia Corporation
+ * Copyright (C) 2005-2008 Nokia Corporation
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * version 2.1 as published by the Free Software Foundation.
+ * This work is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This work is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
+ * License along with this work; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __SIP_CONNECTION_PRIVATE_H__
-#define __SIP_CONNECTION_PRIVATE_H__
+#ifndef __TPSIP_CONNECTION_PRIVATE_H__
+#define __TPSIP_CONNECTION_PRIVATE_H__
 
 #include <telepathy-glib/channel-factory-iface.h>
 
-#include "sip-connection-sofia.h"
+#include <tpsip/sofia-decls.h>
+#include <sofia-sip/sresolv.h>
 
-struct _SIPConnectionPrivate
+struct _TpsipConnectionPrivate
 {
-  SIPConnectionSofia *sofia;
+  su_root_t *sofia_root;
   nua_t  *sofia_nua;
   su_home_t *sofia_home;
   nua_handle_t *register_op;
@@ -47,7 +49,7 @@ struct _SIPConnectionPrivate
   gchar *auth_user;
   gchar *password;
   gchar *transport;
-  SIPConnectionKeepaliveMechanism keepalive_mechanism;
+  TpsipConnectionKeepaliveMechanism keepalive_mechanism;
   gint keepalive_interval;
   gboolean discover_stun;
   gchar *stun_host;
@@ -56,13 +58,14 @@ struct _SIPConnectionPrivate
   guint local_port;
   gchar *extra_auth_user;
   gchar *extra_auth_password;
+  gboolean loose_routing;
   gboolean discover_binding;
 
   gboolean dispose_has_run;
 };
 
-#define SIP_PROTOCOL_STRING               "sip"
+/* #define TPSIP_PROTOCOL_STRING               "sip" */
 
-#define SIP_CONNECTION_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), SIP_TYPE_CONNECTION, SIPConnectionPrivate))
+#define TPSIP_CONNECTION_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), TPSIP_TYPE_CONNECTION, TpsipConnectionPrivate))
 
-#endif /*__SIP_CONNECTION_PRIVATE_H__*/
+#endif /*__TPSIP_CONNECTION_PRIVATE_H__*/
