@@ -1,23 +1,25 @@
 /*
- * sip-text-channel.c - Source for SIPTextChannel
+ * sip-text-channel.c - Source for TpsipTextChannel
  * Copyright (C) 2005-2007 Collabora Ltd.
- * Copyright (C) 2005-2007 Nokia Corporation
- * @author Martti Mela <first.surname@nokia.com>
+ * Copyright (C) 2005-2008 Nokia Corporation
+ *   @author Martti Mela <first.surname@nokia.com>
+ *   @author Mikhail Zabaluev <mikhail.zabaluev@nokia.com>
  *
  * Based on telepathy-gabble implementation (gabble-im-channel).
  *   @author See gabble-im-channel.c
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * version 2.1 as published by the Free Software Foundation.
+ * This work is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This work is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
+ * License along with this work; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
@@ -26,29 +28,38 @@
 #include <string.h>
 #include <time.h>
 
-#include <sofia-sip/sip.h>
-#include <sofia-sip/sip_header.h>
-
 #include <telepathy-glib/channel-iface.h>
-#include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/errors.h>
+#include <telepathy-glib/gtypes.h>
+#include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/svc-channel.h>
 
+#include <tpsip/event-target.h>
+
 #include "sip-text-channel.h"
 
 #include "sip-connection.h"
 #include "sip-connection-helpers.h"
 
-#include "telepathy-helpers.h" 
+#include <sofia-sip/sip.h>
+#include <sofia-sip/sip_header.h>
 
-#define DEBUG_FLAG SIP_DEBUG_IM
+#define DEBUG_FLAG TPSIP_DEBUG_IM
 #include "debug.h"
 
+static gboolean
+tpsip_text_channel_nua_r_message_cb (TpsipTextChannel *self,
+                                     const TpsipNuaEvent *ev,
+                                     tagi_t            tags[],
+                                     gpointer          foo);
+
+static void event_target_iface_init (gpointer, gpointer);
 static void channel_iface_init (gpointer, gpointer);
 static void text_iface_init (gpointer, gpointer);
 
-G_DEFINE_TYPE_WITH_CODE (SIPTextChannel, sip_text_channel, G_TYPE_OBJECT,
+G_DEFINE_TYPE_WITH_CODE (TpsipTextChannel, tpsip_text_channel, G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (TPSIP_TYPE_EVENT_TARGET, event_target_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_TEXT, text_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_IFACE, NULL));
@@ -66,23 +77,11 @@ enum
 };
 
 
-DEFINE_TP_STRUCT_TYPE(sip_tp_pending_message_struct_type,
-                      G_TYPE_UINT,
-                      G_TYPE_UINT,
-                      G_TYPE_UINT,
-                      G_TYPE_UINT,
-                      G_TYPE_UINT,
-                      G_TYPE_STRING)
-
-DEFINE_TP_LIST_TYPE(sip_tp_pending_message_list_type,
-                    sip_tp_pending_message_struct_type ())
-
-
 /* private structures */
 
-typedef struct _SIPTextPendingMessage SIPTextPendingMessage;
+typedef struct _TpsipTextPendingMessage TpsipTextPendingMessage;
 
-struct _SIPTextPendingMessage
+struct _TpsipTextPendingMessage
 {
   guint id;
   nua_handle_t *nh;
@@ -96,11 +95,11 @@ struct _SIPTextPendingMessage
   gchar *text;
 };
 
-typedef struct _SIPTextChannelPrivate SIPTextChannelPrivate;
+typedef struct _TpsipTextChannelPrivate TpsipTextChannelPrivate;
 
-struct _SIPTextChannelPrivate
+struct _TpsipTextChannelPrivate
 {
-  SIPConnection *conn;
+  TpsipConnection *conn;
   gchar *object_path;
   TpHandle handle;
 
@@ -115,32 +114,32 @@ struct _SIPTextChannelPrivate
 };
 
 
-#define _sip_text_pending_new() \
-	(g_slice_new(SIPTextPendingMessage))
-#define _sip_text_pending_new0() \
-	(g_slice_new0(SIPTextPendingMessage))
+#define _tpsip_text_pending_new() \
+	(g_slice_new(TpsipTextPendingMessage))
+#define _tpsip_text_pending_new0() \
+	(g_slice_new0(TpsipTextPendingMessage))
 
-#define SIP_TEXT_CHANNEL_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), SIP_TYPE_TEXT_CHANNEL, SIPTextChannelPrivate))
+#define TPSIP_TEXT_CHANNEL_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), TPSIP_TYPE_TEXT_CHANNEL, TpsipTextChannelPrivate))
 
-static void _sip_text_pending_free(SIPTextPendingMessage *msg)
+static void _tpsip_text_pending_free(TpsipTextPendingMessage *msg)
 {
   if (msg->text)
     g_free (msg->text);
 
   nua_handle_unref (msg->nh);
 
-  g_slice_free (SIPTextPendingMessage, msg);
+  g_slice_free (TpsipTextPendingMessage, msg);
 }
 
-static void _sip_text_pending_free_walk(gpointer data, gpointer user_data)
+static void _tpsip_text_pending_free_walk(gpointer data, gpointer user_data)
 {
-  _sip_text_pending_free ((SIPTextPendingMessage *) data);
+  _tpsip_text_pending_free ((TpsipTextPendingMessage *) data);
 }
 
 static void
-sip_text_channel_init (SIPTextChannel *obj)
+tpsip_text_channel_init (TpsipTextChannel *obj)
 {
-  SIPTextChannelPrivate *priv = SIP_TEXT_CHANNEL_GET_PRIVATE (obj);
+  TpsipTextChannelPrivate *priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE (obj);
 
   DEBUG("enter");
 
@@ -149,21 +148,28 @@ sip_text_channel_init (SIPTextChannel *obj)
 }
 
 static
-GObject *sip_text_channel_constructor(GType type,
+GObject *tpsip_text_channel_constructor(GType type,
 				      guint n_props,
 				      GObjectConstructParam *props)
 {
   GObject *obj;
-  SIPTextChannelPrivate *priv;
+  TpsipTextChannelPrivate *priv;
   DBusGConnection *bus;
 
   DEBUG("enter");
 
   obj =
-    G_OBJECT_CLASS(sip_text_channel_parent_class)->constructor(type,
+    G_OBJECT_CLASS(tpsip_text_channel_parent_class)->constructor(type,
 							       n_props,
 							       props);
-  priv = SIP_TEXT_CHANNEL_GET_PRIVATE(SIP_TEXT_CHANNEL(obj));
+  priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE(TPSIP_TEXT_CHANNEL(obj));
+
+  tpsip_connection_connect_auth_handler (priv->conn, TPSIP_EVENT_TARGET (obj));
+
+  g_signal_connect (obj,
+                    "nua-event::nua_r_message",
+                    G_CALLBACK (tpsip_text_channel_nua_r_message_cb),
+                    NULL);
 
   bus = tp_get_bus();
   dbus_g_connection_register_g_object(bus, priv->object_path, obj);
@@ -172,77 +178,62 @@ GObject *sip_text_channel_constructor(GType type,
 }
 
 
-static void sip_text_channel_get_property(GObject    *object,
+static void tpsip_text_channel_get_property(GObject    *object,
 					  guint       property_id,
 					  GValue     *value,
 					  GParamSpec *pspec);
-static void sip_text_channel_set_property(GObject     *object,
+static void tpsip_text_channel_set_property(GObject     *object,
 					  guint        property_id,
 					  const GValue *value,
 					  GParamSpec   *pspec);
-static void sip_text_channel_dispose(GObject *object);
-static void sip_text_channel_finalize(GObject *object);
+static void tpsip_text_channel_dispose(GObject *object);
+static void tpsip_text_channel_finalize(GObject *object);
 
 static void
-sip_text_channel_class_init(SIPTextChannelClass *sip_text_channel_class)
+tpsip_text_channel_class_init(TpsipTextChannelClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (sip_text_channel_class);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GParamSpec *param_spec;
 
   DEBUG("enter");
 
-  g_type_class_add_private (sip_text_channel_class, sizeof (SIPTextChannelPrivate));
+  g_type_class_add_private (klass, sizeof (TpsipTextChannelPrivate));
 
-  object_class->get_property = sip_text_channel_get_property;
-  object_class->set_property = sip_text_channel_set_property;
+  object_class->get_property = tpsip_text_channel_get_property;
+  object_class->set_property = tpsip_text_channel_set_property;
 
-  object_class->constructor = sip_text_channel_constructor;
+  object_class->constructor = tpsip_text_channel_constructor;
 
-  object_class->dispose = sip_text_channel_dispose;
-  object_class->finalize = sip_text_channel_finalize;
+  object_class->dispose = tpsip_text_channel_dispose;
+  object_class->finalize = tpsip_text_channel_finalize;
 
-  param_spec = g_param_spec_object("connection", "SIPConnection object",
+  param_spec = g_param_spec_object("connection", "TpsipConnection object",
 				   "SIP connection object that owns this "
 				   "SIP media channel object.",
-				   SIP_TYPE_CONNECTION,
+				   TPSIP_TYPE_CONNECTION,
 				   G_PARAM_CONSTRUCT_ONLY |
 				   G_PARAM_READWRITE |
 				   G_PARAM_STATIC_NICK |
 				   G_PARAM_STATIC_BLURB);
   g_object_class_install_property(object_class, PROP_CONNECTION, param_spec);
 
-  param_spec = g_param_spec_string("object-path", "D-Bus object path",
-				   "The D-Bus object path used for this "
-				   "object on the bus.",
-				   NULL,
-				   G_PARAM_CONSTRUCT_ONLY |
-				   G_PARAM_READWRITE |
-				   G_PARAM_STATIC_NAME |
-				   G_PARAM_STATIC_BLURB);
-  g_object_class_install_property(object_class, PROP_OBJECT_PATH, param_spec);
-
-  param_spec = g_param_spec_string("channel-type", "Telepathy channel type",
-				   "The D-Bus interface representing the "
-				   "type of this channel.",
-				   NULL,
-				   G_PARAM_READABLE |
-				   G_PARAM_STATIC_NAME |
-				   G_PARAM_STATIC_BLURB);
-  g_object_class_install_property(object_class, PROP_CHANNEL_TYPE, param_spec);
-
   g_object_class_override_property (object_class, PROP_HANDLE_TYPE,
       "handle-type");
   g_object_class_override_property (object_class, PROP_HANDLE, "handle");
+  g_object_class_override_property (object_class, PROP_OBJECT_PATH,
+      "object-path");
+  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE,
+      "channel-type");
 }
 
-static
-void sip_text_channel_get_property(GObject *object,
+static void
+tpsip_text_channel_get_property(GObject *object,
 				   guint property_id,
 				   GValue *value,
 				   GParamSpec *pspec)
 {
-  SIPTextChannel *chan = SIP_TEXT_CHANNEL(object);
-  SIPTextChannelPrivate *priv = SIP_TEXT_CHANNEL_GET_PRIVATE(chan);
+  TpsipTextChannel *chan = TPSIP_TEXT_CHANNEL(object);
+  TpsipTextChannelPrivate *priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE(chan);
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -271,15 +262,14 @@ void sip_text_channel_get_property(GObject *object,
   }
 }
 
-static
-void
-sip_text_channel_set_property(GObject *object,
+static void
+tpsip_text_channel_set_property(GObject *object,
 			      guint property_id,
 			      const GValue *value,
 			      GParamSpec *pspec)
 {
-  SIPTextChannel *chan = SIP_TEXT_CHANNEL (object);
-  SIPTextChannelPrivate *priv = SIP_TEXT_CHANNEL_GET_PRIVATE (chan);
+  TpsipTextChannel *chan = TPSIP_TEXT_CHANNEL (object);
+  TpsipTextChannelPrivate *priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE (chan);
 
   switch (property_id) {
   case PROP_CONNECTION:
@@ -308,10 +298,10 @@ sip_text_channel_set_property(GObject *object,
 }
 
 static void
-sip_text_channel_dispose(GObject *object)
+tpsip_text_channel_dispose(GObject *object)
 {
-  SIPTextChannel *self = SIP_TEXT_CHANNEL (object);
-  SIPTextChannelPrivate *priv = SIP_TEXT_CHANNEL_GET_PRIVATE (self);
+  TpsipTextChannel *self = TPSIP_TEXT_CHANNEL (object);
+  TpsipTextChannelPrivate *priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE (self);
 
   if (priv->dispose_has_run)
     return;
@@ -319,19 +309,19 @@ sip_text_channel_dispose(GObject *object)
   priv->dispose_has_run = TRUE;
 
   if (!priv->closed)
-    sip_text_channel_close (self);
+    tpsip_text_channel_close (self);
 
   /* release any references held by the object here */
 
-  if (G_OBJECT_CLASS (sip_text_channel_parent_class)->dispose)
-    G_OBJECT_CLASS (sip_text_channel_parent_class)->dispose (object);
+  if (G_OBJECT_CLASS (tpsip_text_channel_parent_class)->dispose)
+    G_OBJECT_CLASS (tpsip_text_channel_parent_class)->dispose (object);
 }
 
 static void
-sip_text_channel_finalize(GObject *object)
+tpsip_text_channel_finalize(GObject *object)
 {
-  SIPTextChannel *self = SIP_TEXT_CHANNEL (object);
-  SIPTextChannelPrivate *priv = SIP_TEXT_CHANNEL_GET_PRIVATE (self);
+  TpsipTextChannel *self = TPSIP_TEXT_CHANNEL (object);
+  TpsipTextChannelPrivate *priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE (self);
 
   DEBUG("enter");
 
@@ -340,7 +330,7 @@ sip_text_channel_finalize(GObject *object)
       g_warning ("zapping %u pending incoming messages",
                  g_queue_get_length (priv->pending_messages));
       g_queue_foreach (priv->pending_messages,
-                       _sip_text_pending_free_walk, NULL);
+                       _tpsip_text_pending_free_walk, NULL);
     }
   g_queue_free (priv->pending_messages);
 
@@ -349,50 +339,50 @@ sip_text_channel_finalize(GObject *object)
       g_message ("zapping %u pending outgoing message requests",
                  g_queue_get_length (priv->messages_to_be_acknowledged));
       g_queue_foreach (priv->messages_to_be_acknowledged,
-                       _sip_text_pending_free_walk, NULL);
+                       _tpsip_text_pending_free_walk, NULL);
     }
   g_queue_free (priv->messages_to_be_acknowledged);
 
   g_free (priv->object_path);
 
-  G_OBJECT_CLASS (sip_text_channel_parent_class)->finalize (object);
+  G_OBJECT_CLASS (tpsip_text_channel_parent_class)->finalize (object);
 }
 
-static gint sip_pending_message_compare(gconstpointer msg, gconstpointer id)
+static gint tpsip_pending_message_compare(gconstpointer msg, gconstpointer id)
 {
-  SIPTextPendingMessage *message = (SIPTextPendingMessage *)(msg);
+  TpsipTextPendingMessage *message = (TpsipTextPendingMessage *)(msg);
   return (message->id != GPOINTER_TO_INT(id));
 }
 
-static gint sip_acknowledged_messages_compare(gconstpointer msg,
+static gint tpsip_acknowledged_messages_compare(gconstpointer msg,
 					      gconstpointer id)
 {
-  SIPTextPendingMessage *message = (SIPTextPendingMessage *)msg;
+  TpsipTextPendingMessage *message = (TpsipTextPendingMessage *)msg;
   nua_handle_t *nh = (nua_handle_t *) id;
   return (message->nh != nh);
 }
 
 /**
- * sip_text_channel_acknowledge_pending_messages
+ * tpsip_text_channel_acknowledge_pending_messages
  *
  * Implements DBus method AcknowledgePendingMessages
  * on interface org.freedesktop.Telepathy.Channel.Type.Text
  */
 static void
-sip_text_channel_acknowledge_pending_messages(TpSvcChannelTypeText *iface,
+tpsip_text_channel_acknowledge_pending_messages(TpSvcChannelTypeText *iface,
 					      const GArray *ids,
 					      DBusGMethodInvocation *context)
 {
-  SIPTextChannel *obj = SIP_TEXT_CHANNEL (iface);
-  SIPTextChannelPrivate *priv;
+  TpsipTextChannel *obj = TPSIP_TEXT_CHANNEL (iface);
+  TpsipTextChannelPrivate *priv;
   TpHandleRepoIface *contact_repo;
   GList **nodes;
-  SIPTextPendingMessage *msg;
+  TpsipTextPendingMessage *msg;
   guint i;
 
   DEBUG("enter");
   
-  priv = SIP_TEXT_CHANNEL_GET_PRIVATE(obj);
+  priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE(obj);
   contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *)(priv->conn), TP_HANDLE_TYPE_CONTACT);
 
@@ -404,7 +394,7 @@ sip_text_channel_acknowledge_pending_messages(TpSvcChannelTypeText *iface,
 
       nodes[i] = g_queue_find_custom (priv->pending_messages,
                                       GINT_TO_POINTER (id),
-                                      sip_pending_message_compare);
+                                      tpsip_pending_message_compare);
 
       if (nodes[i] == NULL)
         {
@@ -424,14 +414,14 @@ sip_text_channel_acknowledge_pending_messages(TpSvcChannelTypeText *iface,
 
   for (i = 0; i < ids->len; i++)
     {
-      msg = (SIPTextPendingMessage *) nodes[i]->data;
+      msg = (TpsipTextPendingMessage *) nodes[i]->data;
 
       DEBUG("acknowledging message id %u", msg->id);
 
       g_queue_remove (priv->pending_messages, msg);
 
       tp_handle_unref (contact_repo, msg->sender);
-      _sip_text_pending_free (msg);
+      _tpsip_text_pending_free (msg);
     }
 
   g_free(nodes);
@@ -441,27 +431,27 @@ sip_text_channel_acknowledge_pending_messages(TpSvcChannelTypeText *iface,
 
 
 /**
- * sip_text_channel_close_async
+ * tpsip_text_channel_close_async
  *
  * Implements DBus method Close
  * on interface org.freedesktop.Telepathy.Channel
  */
 static void
-sip_text_channel_close_async (TpSvcChannel *iface,
+tpsip_text_channel_close_async (TpSvcChannel *iface,
                               DBusGMethodInvocation *context)
 {
-  sip_text_channel_close (SIP_TEXT_CHANNEL(iface));
+  tpsip_text_channel_close (TPSIP_TEXT_CHANNEL(iface));
   tp_svc_channel_return_from_close (context);
 }
 
 void
-sip_text_channel_close (SIPTextChannel *self)
+tpsip_text_channel_close (TpsipTextChannel *self)
 {
-  SIPTextChannelPrivate *priv;
+  TpsipTextChannelPrivate *priv;
 
   DEBUG("enter");
 
-  priv = SIP_TEXT_CHANNEL_GET_PRIVATE(self);
+  priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE(self);
   if (!priv->closed)
     {
       priv->closed = TRUE;
@@ -470,13 +460,13 @@ sip_text_channel_close (SIPTextChannel *self)
 }
 
 /**
- * sip_text_channel_get_channel_type
+ * tpsip_text_channel_get_channel_type
  *
  * Implements DBus method GetChannelType
  * on interface org.freedesktop.Telepathy.Channel
  */
 static void
-sip_text_channel_get_channel_type (TpSvcChannel *iface,
+tpsip_text_channel_get_channel_type (TpSvcChannel *iface,
                                    DBusGMethodInvocation *context)
 {
   DEBUG("enter");
@@ -487,21 +477,21 @@ sip_text_channel_get_channel_type (TpSvcChannel *iface,
 
 
 /**
- * sip_text_channel_get_handle
+ * tpsip_text_channel_get_handle
  *
  * Implements DBus method GetHandle
  * on interface org.freedesktop.Telepathy.Channel
  */
 static void
-sip_text_channel_get_handle (TpSvcChannel *iface,
+tpsip_text_channel_get_handle (TpSvcChannel *iface,
                              DBusGMethodInvocation *context)
 {
-  SIPTextChannel *obj = SIP_TEXT_CHANNEL (iface);
-  SIPTextChannelPrivate *priv;
+  TpsipTextChannel *obj = TPSIP_TEXT_CHANNEL (iface);
+  TpsipTextChannelPrivate *priv;
 
   DEBUG("enter");
 
-  priv = SIP_TEXT_CHANNEL_GET_PRIVATE(obj);
+  priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE(obj);
 
   tp_svc_channel_return_from_get_handle (context, TP_HANDLE_TYPE_CONTACT,
       priv->handle);
@@ -509,13 +499,13 @@ sip_text_channel_get_handle (TpSvcChannel *iface,
 
 
 /**
- * sip_text_channel_get_interfaces
+ * tpsip_text_channel_get_interfaces
  *
  * Implements DBus method GetInterfaces
  * on interface org.freedesktop.Telepathy.Channel
  */
 static void
-sip_text_channel_get_interfaces(TpSvcChannel *iface,
+tpsip_text_channel_get_interfaces(TpSvcChannel *iface,
                                 DBusGMethodInvocation *context)
 {
   const char *interfaces[] = { NULL };
@@ -526,13 +516,13 @@ sip_text_channel_get_interfaces(TpSvcChannel *iface,
 
 
 /**
- * sip_text_channel_get_message_types
+ * tpsip_text_channel_get_message_types
  *
  * Implements DBus method GetMessageTypes
  * on interface org.freedesktop.Telepathy.Channel.Type.Text
  */
 static void
-sip_text_channel_get_message_types(TpSvcChannelTypeText *iface,
+tpsip_text_channel_get_message_types(TpSvcChannelTypeText *iface,
                                    DBusGMethodInvocation *context)
 {
   GArray *ret = g_array_sized_new (FALSE, FALSE, sizeof (guint), 1);
@@ -545,12 +535,12 @@ sip_text_channel_get_message_types(TpSvcChannelTypeText *iface,
 }
 
 static void
-sip_pending_message_list_add (GPtrArray *list, SIPTextPendingMessage *msg)
+tpsip_pending_message_list_add (GPtrArray *list, TpsipTextPendingMessage *msg)
 {
   GValue val = { 0 };
   GType message_type;
 
-  message_type = sip_tp_pending_message_struct_type ();
+  message_type = TP_STRUCT_TYPE_PENDING_TEXT_MESSAGE;
   g_value_init (&val, message_type);
   g_value_take_boxed (&val,
                       dbus_g_type_specialized_construct (message_type));
@@ -567,24 +557,24 @@ sip_pending_message_list_add (GPtrArray *list, SIPTextPendingMessage *msg)
 }
 
 /**
- * sip_text_channel_list_pending_messages
+ * tpsip_text_channel_list_pending_messages
  *
  * Implements DBus method ListPendingMessages
  * on interface org.freedesktop.Telepathy.Channel.Type.Text
  */
 static void
-sip_text_channel_list_pending_messages(TpSvcChannelTypeText *iface,
+tpsip_text_channel_list_pending_messages(TpSvcChannelTypeText *iface,
                                        gboolean clear,
                                        DBusGMethodInvocation *context)
 {
-  SIPTextChannel *self = (SIPTextChannel*) iface;
-  SIPTextChannelPrivate *priv;
+  TpsipTextChannel *self = (TpsipTextChannel*) iface;
+  TpsipTextChannelPrivate *priv;
   GPtrArray *messages;
   GList *cur;
   guint count;
 
-  g_assert (SIP_IS_TEXT_CHANNEL(self));
-  priv = SIP_TEXT_CHANNEL_GET_PRIVATE (self);
+  g_assert (TPSIP_IS_TEXT_CHANNEL(self));
+  priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE (self);
 
   count = g_queue_get_length (priv->pending_messages);
   messages = g_ptr_array_sized_new (count);
@@ -592,27 +582,27 @@ sip_text_channel_list_pending_messages(TpSvcChannelTypeText *iface,
   if (clear)
     {
       while ((cur = g_queue_pop_head_link (priv->pending_messages)) != NULL)
-        sip_pending_message_list_add (messages,
-                                      (SIPTextPendingMessage *) cur->data);
+        tpsip_pending_message_list_add (messages,
+                                        (TpsipTextPendingMessage *) cur->data);
     }
   else
     {
       for (cur = g_queue_peek_head_link(priv->pending_messages);
            cur != NULL;
            cur = cur->next)
-        sip_pending_message_list_add (messages,
-                                      (SIPTextPendingMessage *) cur->data);
+        tpsip_pending_message_list_add (messages,
+                                        (TpsipTextPendingMessage *) cur->data);
     }
 
   tp_svc_channel_type_text_return_from_list_pending_messages (context,
       messages);
 
-  g_boxed_free (sip_tp_pending_message_list_type (), messages);
+  g_boxed_free (TP_ARRAY_TYPE_PENDING_TEXT_MESSAGE_LIST, messages);
 }
 
 
 /**
- * sip_text_channel_send
+ * tpsip_text_channel_send
  *
  * Implements DBus method Send
  * on interface org.freedesktop.Telepathy.Channel.Type.Text
@@ -624,14 +614,14 @@ sip_text_channel_list_pending_messages(TpSvcChannelTypeText *iface,
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
 static void
-sip_text_channel_send(TpSvcChannelTypeText *iface,
-                      guint type,
-                      const gchar *text,
-                      DBusGMethodInvocation *context)
+tpsip_text_channel_send(TpSvcChannelTypeText *iface,
+                        guint type,
+                        const gchar *text,
+                        DBusGMethodInvocation *context)
 {
-  SIPTextChannel *self = SIP_TEXT_CHANNEL(iface);
-  SIPTextChannelPrivate *priv = SIP_TEXT_CHANNEL_GET_PRIVATE (self);
-  SIPTextPendingMessage *msg = NULL;
+  TpsipTextChannel *self = TPSIP_TEXT_CHANNEL(iface);
+  TpsipTextChannelPrivate *priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE (self);
+  TpsipTextPendingMessage *msg = NULL;
   nua_handle_t *msg_nh = NULL;
 
   DEBUG("enter");
@@ -639,68 +629,79 @@ sip_text_channel_send(TpSvcChannelTypeText *iface,
   if (priv->handle == 0)
     {
       GError invalid =  { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-          "invalid recipient" };
+          "Invalid recipient" };
 
       g_warning ("invalid recipient handle %d", priv->handle);
       dbus_g_method_return_error (context, &invalid);
       return;
     }
 
-  /* XXX: would it be helpful to bind the channel, or the
-   * SIPTextPendingMessage, or something, to the NH? */
+  msg_nh = tpsip_conn_create_request_handle (priv->conn, priv->handle);
+  if (msg_nh == NULL)
+    {
+      GError e = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          "Request creation failed" };
+      dbus_g_method_return_error (context, &e);
+      return;
+    }
 
-  msg_nh = sip_conn_create_request_handle (priv->conn, priv->handle);
-  g_assert (msg_nh != NULL);
+  tpsip_event_target_attach (msg_nh, (GObject *) self);
 
   nua_message(msg_nh,
 	      SIPTAG_CONTENT_TYPE_STR("text/plain"),
 	      SIPTAG_PAYLOAD_STR(text),
 	      TAG_END());
 
-  msg = _sip_text_pending_new();
+  msg = _tpsip_text_pending_new();
   msg->nh = msg_nh;
   msg->text = g_strdup(text);
   msg->type = type;
   msg->timestamp = time(NULL);
 
-  g_queue_push_tail(priv->messages_to_be_acknowledged, msg);
+  g_queue_push_tail (priv->messages_to_be_acknowledged, msg);
+
+  DEBUG("message queued for delivery with timestamp %u", (guint)msg->timestamp);
 
   tp_svc_channel_type_text_return_from_send (context);
 }
 
-void
-sip_text_channel_emit_message_status(SIPTextChannel *obj,
-                                     nua_handle_t *nh,
-                                     int status)
+static gboolean
+tpsip_text_channel_nua_r_message_cb (TpsipTextChannel *self,
+                                     const TpsipNuaEvent *ev,
+                                     tagi_t            tags[],
+                                     gpointer          foo)
 {
-  SIPTextChannelPrivate *priv = SIP_TEXT_CHANNEL_GET_PRIVATE (obj);
-  SIPTextPendingMessage *msg;
+  TpsipTextChannelPrivate *priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE (self);
+  TpsipTextPendingMessage *msg;
   TpChannelTextSendError send_error;
   GList *node;
 
   DEBUG("enter");
 
-  node = g_queue_find_custom(priv->messages_to_be_acknowledged, nh,
-			     sip_acknowledged_messages_compare);
+  node = g_queue_find_custom (priv->messages_to_be_acknowledged,
+                              ev->nua_handle,
+			      tpsip_acknowledged_messages_compare);
 
   /* Shouldn't happen... */
-  if (!node) {
-    g_warning ("message not found");
-    return;
-  }
-  
-  msg = (SIPTextPendingMessage *)node->data;
-
-  g_return_if_fail (msg != NULL);
-
-  if (status >= 200 && status < 300)
+  if (node == NULL)
     {
-      tp_svc_channel_type_text_emit_sent ((TpSvcChannelTypeText *)obj,
+      g_warning ("message pending sent acknowledgement not found");
+      return FALSE;
+    }
+
+  msg = (TpsipTextPendingMessage *)node->data;
+
+  g_assert (msg != NULL);
+
+  if (ev->status >= 200 && ev->status < 300)
+    {
+      DEBUG("message with timestamp %u delivered", (guint)msg->timestamp);
+      tp_svc_channel_type_text_emit_sent (self,
           msg->timestamp, msg->type, msg->text);
     }
   else
     {
-      switch (status)
+      switch (ev->status)
         {
         case 401:
         case 403:
@@ -736,33 +737,33 @@ sip_text_channel_emit_message_status(SIPTextChannel *obj,
           send_error = TP_CHANNEL_TEXT_SEND_ERROR_UNKNOWN;
       }
 
-    DEBUG("emitting send error %d %s", (int)send_error, msg->text);
-
-    tp_svc_channel_type_text_emit_send_error ((TpSvcChannelTypeText *)obj,
+    tp_svc_channel_type_text_emit_send_error (self,
 	send_error, msg->timestamp, msg->type, msg->text);  
   }
 
   g_queue_remove(priv->messages_to_be_acknowledged, msg);
-  _sip_text_pending_free(msg);
+  _tpsip_text_pending_free(msg);
+
+  return TRUE;
 }
 
-void sip_text_channel_receive(SIPTextChannel *chan,
+void tpsip_text_channel_receive(TpsipTextChannel *chan,
 			      TpHandle sender,
 			      const char *message)
 {
-  SIPTextPendingMessage *msg;
-  SIPTextChannelPrivate *priv;
+  TpsipTextPendingMessage *msg;
+  TpsipTextChannelPrivate *priv;
   TpHandleRepoIface *contact_repo;
 
   DEBUG("enter");
 
   g_assert(chan != NULL);
-  g_assert(SIP_IS_TEXT_CHANNEL(chan));
-  priv = SIP_TEXT_CHANNEL_GET_PRIVATE (chan);
+  g_assert(TPSIP_IS_TEXT_CHANNEL(chan));
+  priv = TPSIP_TEXT_CHANNEL_GET_PRIVATE (chan);
   contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *)(priv->conn), TP_HANDLE_TYPE_CONTACT);
 
-  msg = _sip_text_pending_new();
+  msg = _tpsip_text_pending_new();
 
   msg->id = priv->recv_id++;
   msg->timestamp = time(NULL);
@@ -781,13 +782,18 @@ void sip_text_channel_receive(SIPTextChannel *chan,
       0 /* flags */, msg->text);
 }
 
+static void
+event_target_iface_init (gpointer g_iface, gpointer iface_data)
+{
+}
+
 static void
 channel_iface_init(gpointer g_iface, gpointer iface_data)
 {
   TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
 
 #define IMPLEMENT(x, suffix) tp_svc_channel_implement_##x (\
-    klass, sip_text_channel_##x##suffix)
+    klass, tpsip_text_channel_##x##suffix)
   IMPLEMENT(close,_async);
   IMPLEMENT(get_channel_type,);
   IMPLEMENT(get_handle,);
@@ -801,7 +807,7 @@ text_iface_init (gpointer g_iface, gpointer iface_data)
   TpSvcChannelTypeTextClass *klass = (TpSvcChannelTypeTextClass *)g_iface;
 
 #define IMPLEMENT(x) tp_svc_channel_type_text_implement_##x (klass,\
-    sip_text_channel_##x)
+    tpsip_text_channel_##x)
   IMPLEMENT(acknowledge_pending_messages);
   IMPLEMENT(get_message_types);
   IMPLEMENT(list_pending_messages);
